
/*
 * Lab G: Complete version of BPC
 * Steps 1 to 10.
 */

/*
 * BusinessProcessCoordinator.java
 * DESCRIPTION : Acts as Business Process Manager by coordinating following activites
 *               1. Receives Message From Servlet
 *               2. Sends asynchronous request to Credit Check Service
 *               3. Sends asynchronous request to Inventory Check Service
 *               4. When both the replies are received, sends reply back to servlet
 *
 */

import org.w3c.dom.Element;
import java.util.Hashtable;
import com.tibco.tibchoc.*;

// Step 1: Import the entire javax.jms package
import javax.jms.*;

public class BusinessProcessCoordinator implements MessageListener
{
    private String _propertiesFileName;
    private String _serverName;
    private String _debug;
    private String _user;
    private String _password;
    private String _factory;

    private ConnectionFactory  _Factory;
    private Connection _Connection;

    private Session    _Session1;
    private Session    _Session2;
    private Session    _Session3;
    private Session    _Session4;

    private Destination     _webReceiveDestination,
                            _webReplyDestination,
                            _inventoryCheckDestination,
                            _creditCheckDestination,
                            _replyInventoryCheckDestination,
                            _replyCreditCheckDestination;

    private MessageConsumer   _webReceiver;
    private MessageProducer   _webReplier,
                            _inventoryCheckProducer,
                            _creditCheckProducer;

    private MessageProducer  _orderEntryProducer,
                            _inventoryUpdateProducer;


    public BusinessProcessCoordinator (String fileName) throws Exception
    {
        _propertiesFileName = fileName;
    }

    public void init() throws Exception
    {
        System.out.println("Business Process Manager : init ");
        AppProperties.loadProperties(_propertiesFileName);
        _serverName    = AppProperties.getProperty("TIBCHOC.serverName");
        _debug         = AppProperties.getProperty("TIBCHOC.debug");
        _user          = AppProperties.getProperty("TIBCHOC.user");
        _password      = AppProperties.getProperty("TIBCHOC.password");
        _factory       = AppProperties.getProperty("TIBCHOC.factory");

        openJMSConnection();
    }

    public void initialize()
    {
        //initialization of Connection Factory and creation of connection
        try
        {
            TibjmsUtilities.initJNDI(_serverName);

            // Step 2: Use the JNDI to obtain a connection factory
            _Factory = (ConnectionFactory)TibjmsUtilities.lookup(_factory);

            // Step 3: Create a connnection using the connection factory
            _Connection = _Factory.createConnection(_user,_password);

            // Step 4: Create a few sessions
            // a) session to handle the order connection from the servlet
            _Session1 = _Connection.createSession(false,Session.AUTO_ACKNOWLEDGE);

            // b) session to handle the inventory queries
            _Session2 = _Connection.createSession(false,Session.AUTO_ACKNOWLEDGE);

            // c) session to handle the credit validation
            _Session3 = _Connection.createSession(false,Session.AUTO_ACKNOWLEDGE);


            // Step 5: Create a session connection (to handle updates for both inventory and subOrder systems)
            _Session4 = _Connection.createSession(true,Session.AUTO_ACKNOWLEDGE);

        }
        catch (Exception e)
        {
            System.out.println(e.toString());
            closeAll();
            System.exit(0);
        }
    }

    public void openJMSConnection()
    {
        try
        {
            System.out.println("JBPC : openJMSConnection ");
            initialize();

            //To receive messages from servlet and reply back

            // Step 6: Obtain a specific destination (Servlet & BPC)
            _webReceiveDestination = (Destination)TibjmsUtilities.lookup(AppProperties.getProperty("TIBCHOC.WEB2JBPC"));

            // Step 7: Create a receiver to use the callback in this class
            _webReceiver    = _Session1.createConsumer(_webReceiveDestination);
            _webReceiver.setMessageListener(this);

            // Step 8: Create a sender
            _webReplier     = _Session1.createProducer(null);
            System.out.println("JBPC : Receiver created : WEB2BPM : "+ AppProperties.getProperty("TIBCHOC.WEB2JBPC"));



            //setup for inventory check producer and consumer for reply
            _inventoryCheckDestination      = (Destination)TibjmsUtilities.lookup(AppProperties.getProperty("TIBCHOC.JBPC2IC"));
            _inventoryCheckProducer    =  _Session2.createProducer(_inventoryCheckDestination);
            _replyInventoryCheckDestination = (Destination)TibjmsUtilities.lookup(AppProperties.getProperty("TIBCHOC.JBPC2ICR"));
            MessageConsumer inventoryReplyReceiver = _Session2.createConsumer(_replyInventoryCheckDestination);
            inventoryReplyReceiver.setMessageListener(new InventoryReplyMsgListener(this));
            System.out.println("JBPC : Inventory Check : Sender created :  "+ AppProperties.getProperty("TIBCHOC.JBPC2IC"));
            System.out.println("JBPC : Inventory Check : Receiver created :  "+ AppProperties.getProperty("TIBCHOC.JBPC2ICR"));



            //setup for credit check producer and consumer for reply
            _creditCheckDestination      = (Destination)TibjmsUtilities.lookup(AppProperties.getProperty("TIBCHOC.JBPC2CC"));
            _creditCheckProducer  = _Session3.createProducer(_creditCheckDestination);
            _replyCreditCheckDestination = (Destination)TibjmsUtilities.lookup(AppProperties.getProperty("TIBCHOC.JBPC2CCR"));
            MessageConsumer replyCreditReceiver = _Session3.createConsumer(_replyCreditCheckDestination);
            replyCreditReceiver.setMessageListener(new CreditReplyMsgListener(this));
            System.out.println("JBPC : Credit Check : Sender created :  "+ AppProperties.getProperty("TIBCHOC.JBPC2CC"));
            System.out.println("JBPC : Credit Check : Receiver created :  "+ AppProperties.getProperty("TIBCHOC.JBPC2CCR"));

            //setup inventory update producer
            String invuDestinationName      = AppProperties.getProperty("TIBCHOC.INVUPDATE");
            Destination _inventoryUpdateDestination = (Destination)TibjmsUtilities.lookup(invuDestinationName);
            _inventoryUpdateProducer  = _Session4.createProducer(_inventoryUpdateDestination);
            System.out.println("JBPC : Producer created for Destination : " + invuDestinationName);

            //setup order entry producer
            String ordDestinationName    = AppProperties.getProperty("TIBCHOC.ORDERENTRY");
            Destination _orderEntryDestination = (Destination)TibjmsUtilities.lookup(ordDestinationName);
            _orderEntryProducer  = _Session4.createProducer(_orderEntryDestination);
            System.out.println("JBPC : Producer created for Destination : " + ordDestinationName);

            // Step 9: Start the connection
            _Connection.start();


            synchronized (this)
            {
            try
            {
                this.wait();
            }
            catch (InterruptedException ie) {}
            }

        }
        catch(Exception e)
        {
            System.out.println(e.toString());
            closeAll();
            System.exit(0);
        }
    }


    public void onMessage(Message message)
    {
        System.out.println("JBPC : Received Message From Servlet");
        if(_debug.equalsIgnoreCase("TRUE"))
            System.out.println("Message :  "+ message.toString());

        try
        {
            //Parse message
            DomDoc msgFromServlet = new DomDoc(((TextMessage)message).getText());
            msgFromServlet.parse();
            Destination retDestination = (Destination)message.getJMSReplyTo();
            String orderid = msgFromServlet.get("OrderID");
            System.out.println("JBPC : Order Id :  "+ orderid);

            InMemoryStructure memStruct =
                    new InMemoryStructure(((TextMessage)message).getText(), retDestination);
            double crReq = new Double(msgFromServlet.get("TotalCharge")).doubleValue();
            memStruct.setRequiredCredit(crReq);
            synchronized(this)
            {
                Hashtable h = OrderHash.getHashInstance();
                h.put(orderid, memStruct);
            }

            //send aynchronous request to credit check
            sendMessageToCreditCheck(msgFromServlet);

            //send aynchronous request to inventory check
            sendMessageToInventoryCheck(msgFromServlet);
        }
        catch(Exception e)
        {
            System.out.println(e.toString());
            closeAll();
            System.exit(0);
        }
    }


    public void sendMessageToCreditCheck(DomDoc msgFromServlet)
    {
        try
        {
            //Do async req-reply to credit check
            //create xml text for message
            TextMessage cMessage = _Session3.createTextMessage();
            DomDocOut cDom = new DomDocOut();
            cDom.createRoot("Order");
            String orderid = msgFromServlet.get("OrderID");
            cDom.add("OrderID", orderid);
            Element parent = cDom.add("CreditInformation");
            cDom.add(parent, "CCNumber", msgFromServlet.get("CCNumber"));
            cDom.add(parent, "CCType",   msgFromServlet.get("CCType"));
            cDom.add(parent, "TotalCharge",   msgFromServlet.get("TotalCharge"));
            cMessage.setText(cDom.toString());
            cMessage.setJMSReplyTo(_replyCreditCheckDestination);
            //send message
            System.out.println("JBPC : Sending msg to credit Check");
            if(_debug.equalsIgnoreCase("TRUE"))
                System.out.println("Message : " + cMessage.toString());

            _creditCheckProducer.send(cMessage);
        }
        catch (Exception e)
        {
            System.out.println("Inside sendMessageToCreditCheck : " + e.toString());
            closeAll();
            System.exit(0);
        }
    }

    public void sendMessageToInventoryCheck(DomDoc msgFromServlet)
    {
        try
        {
            //Do async req-reply to inventory check
            //create xml text for message
            TextMessage iMessage = _Session2.createTextMessage();
            DomDocOut iDom = new DomDocOut();
            iDom.createRoot("Order");
            String orderid = msgFromServlet.get("OrderID");
            iDom.add("OrderID", orderid);
            iDom.importNode(msgFromServlet.getNode("OrderItems"), true );
            iMessage.setText(iDom.toString());
            iMessage.setJMSReplyTo(_replyInventoryCheckDestination);
            //send message
            System.out.println("JBPC : Sending msg to inventory Check");
            if(_debug.equalsIgnoreCase("TRUE"))
                System.out.println("Message : " + iMessage.toString());

            _inventoryCheckProducer.send(iMessage);

        }
        catch (Exception e)
        {
            System.out.println("Inside sendMessageToInventoryCheck : " + e.toString());
            closeAll();
            System.exit(0);
        }
    }

    public void sendReplyToWeb(String id)
    {
        if(_debug.equalsIgnoreCase("TRUE"))
                System.out.println("JBPC : Inside sendReplyToWeb ");
        //create reply message
        try
        {
            Hashtable h = OrderHash.getHashInstance();
            InMemoryStructure in = (InMemoryStructure)h.get(id);
            String retStatus = new String();
            String retDetails = new String();
            if (in.getCreditStatus().equalsIgnoreCase("UNSUCCESS")){
                retStatus = "UNSUCCESS";
                retDetails = in.getCreditDetails();
        }
        else if(in.getInventoryStatus().equalsIgnoreCase("UNSUCCESS"))
        {
            retStatus = "UNSUCCESS";
            retDetails = "The Following Items Are Not In Store To Fulfill Your Order : \n";
            retDetails = retDetails + in.getInventoryDetails() ;
        }
        else
        {
            int ret = sendTMsgOrderInv(id);
            if(ret == 0)
                retStatus = "SUCCESS";
            else
                retStatus = "UNSUCCESS";
        }
        DomDocOut webReplyDoc = new DomDocOut();
        webReplyDoc.createRoot("Order");
        webReplyDoc.add("OrderID", id);
        webReplyDoc.add("Status", retStatus);
        webReplyDoc.add("Detail", retDetails);
        TextMessage wMessage = _Session1.createTextMessage();
        wMessage.setText(webReplyDoc.toString());
        //send reply to servlet
        Destination replyToDestination = in.getReplyToDestination();
        if(_debug.equalsIgnoreCase("TRUE"))
        {
            System.out.println("Reply to web : DestinationName : " + replyToDestination.toString());
            System.out.println("Reply to web : " + webReplyDoc.toString());
        }
        System.out.println("JBPC : Replying to Servlet" + id);

        _webReplier.send(replyToDestination, wMessage);

        }
        catch(Exception e)
        {
            System.out.println(e.toString());
            closeAll();
            System.exit(0);
        }
    }

    public int sendTMsgOrderInv(String id)
    {
        InMemoryStructure memStruct;
        try
        {
            if(_debug.equalsIgnoreCase("TRUE"))
                    System.out.println("JBPC : inside sendTMsgOrderInv ");
            Hashtable hash = OrderHash.getHashInstance();
            memStruct = (InMemoryStructure)hash.get(id);
            DomDoc doc = new DomDoc(memStruct.getXmlMessage());
            doc.parse();

            //create message for OrderSystem
            TextMessage oMessage = _Session4.createTextMessage();
            oMessage.setText(memStruct.getXmlMessage());

            //create message for Inventory Update
            DomDocOut iDom = new DomDocOut();
            iDom.createRoot("OrderID");
            iDom.add("OrderID", id);
            iDom.importNode(doc.getNode("OrderItems"), true );
            TextMessage iMessage = _Session4.createTextMessage();
            iMessage.setText(iDom.toString());

            //send messages
            _orderEntryProducer.send(oMessage);
            _inventoryUpdateProducer.send(iMessage);
            _Session4.commit();

            System.out.println("JBPC : Sent Transacted Message To : OrderEntry & InventoryUpdate");
            if(_debug.equalsIgnoreCase("TRUE"))
            {
                System.out.println("Message to Order Entry : " + oMessage.toString());
                System.out.println("Message to Inventory Update : " + iMessage.toString());
            }
            return 0;
        }
        catch (Exception e)
        {
            System.out.println(e.toString());
            closeAll();
            return 1;
        }
    }

    private void closeAll()
    {
        try
        {
            if (_Connection!=null)
            {
                // Step 10: Close the connection
                _Connection.close();
                _Connection = null;
            }
        }
        catch(JMSException e)
        {
            System.out.println(e.toString());
        }
    }

    public static void main (String[] args)
    {
        try
        {
            BusinessProcessCoordinator bpm = new BusinessProcessCoordinator(args[0]);
            bpm.init();
        }
        catch (Exception e)
        {
            System.out.println(e.toString());
            System.exit(0);
        }
    }
}
