/* 
 * Lab F: Credit Check adapter
 * Steps 1 to 11.
 */

/*
 * CreditCheckAdapter.java
 * DESCRIPTION : Interface between the JMS BPM and Credit System (ie. a flat file) to provide
 *               validation against a credit card and its credit limit of a customer request by
 *               performing the following steps:
 *               Receives message on Destination 
 *               Parses the input xml message
 *               Connect to a Credit System and validate a customers' credit card
 *               Creates XML JMS message from the result of the credit card validation
 *               Sends reply back to replyDest   
 *
 */

import com.tibco.tibchoc.*;

// Step 1: Import the entire javax.jms package
import javax.jms.*;

public class CreditCheckAdapter
{
    private String _propertiesFilename;
    private String _serverName;
    private String _debug;
    private String _receiveDestinationName;
    private String _user;
    private String _password;
    private String _factory;

    private CreditUtil _creditSystem = null;
    private ConnectionFactory _Factory;
    private Connection _Connection;
    private Session _Session;

    private MessageConsumer _Receiver;
    private MessageProducer _replySender;
    private TextMessage _message;
    
    public CreditCheckAdapter(String fileName)
    {
        _propertiesFilename = fileName;
    }
    
    public void init() throws Exception
    {
        AppProperties.loadProperties(_propertiesFilename);
        _serverName             = AppProperties.getProperty("TIBCHOC.serverName");
        _debug                  = AppProperties.getProperty("TIBCHOC.debug");
        _receiveDestinationName = AppProperties.getProperty("TIBCHOC.receiveDestination");
        _user                   = AppProperties.getProperty("TIBCHOC.user");
        _password               = AppProperties.getProperty("TIBCHOC.password");
        _factory                = AppProperties.getProperty("TIBCHOC.factory");
        _creditSystem           = new CreditUtil(AppProperties.getProperty("TIBCHOC.creditFilename"));
        openJMSConnection();
    }
    
    public void openJMSConnection() throws Exception, JMSException
    {
        System.out.println("JMS Credit CheckAdapter : OpenJMSConnection : [" + _serverName + "] Starting up....");
        System.out.println("JMS Credit CheckAdapter : Receive destination name : " + _receiveDestinationName);

        // Using JNDI to connect to the JMS server.
        TibjmsUtilities.initJNDI(_serverName);
        
        // Step 2: Use JNDI to obtain a connection factory
        _Factory = (ConnectionFactory)TibjmsUtilities.lookup("QueueConnectionFactory");
        
        // Step 3: Create a connection using the connection factory
        _Connection = _Factory.createConnection(_user,_password);
        
        // Step 4: Create a session using the connection
        _Session = _Connection.createSession(false,Session.AUTO_ACKNOWLEDGE);

        // Setup a Message Consumer.
        
        // Step 5: Obtain a specific destination
        Destination receiveDest = (Destination)TibjmsUtilities.lookup(_receiveDestinationName);

        // Step 6: Create a Message Consumer
        _Receiver = _Session.createConsumer(receiveDest);

        // Setup a Message Producer.
        
        // Step 7: Create a message Producer
        _replySender = _Session.createProducer(null);
                        
        // Step 8: Start a connection
        _Connection.start();

        for(;;) receiveAndReply();
    }

    public void receiveAndReply() throws Exception
    {
        if(_debug.equalsIgnoreCase("TRUE"))
            System.out.println("JMS Credit CheckAdapter : Inside receiveAndReply : Waiting for messages");

        // Step 9: Receive message
        TextMessage message =  (TextMessage)_Receiver.receive();
        System.out.println("JMS Credit CheckAdapter : Received message" );
            
        if(_debug.equalsIgnoreCase("TRUE"))
            System.out.println("MESSAGE : " + message);
            
            
        if (message.getJMSReplyTo() == null)
			System.out.println("JMS Credit CheckAdapter : Error : ReplyTo is null");
		else
		{
			// Get credit card number from incoming message
            DomDoc msgPBM = new DomDoc(((TextMessage)message).getText());
            msgPBM.parse();
            
            String creditCardNumber = msgPBM.get("CCNumber");
            String creditCardType = msgPBM.get("CCType");
            String orderid = msgPBM.get("OrderID");
            double totalCharge = Double.parseDouble(msgPBM.get("TotalCharge"));

            if(_debug.equalsIgnoreCase("TRUE"))
                System.out.println("Parameters for checkCreditCard(): \n1. orderID "+orderid +"\n2. creditCardNumber " + 
                                    creditCardNumber + "\n3. creditCardType "+creditCardType +"\n4. totalCharge "+totalCharge);

		// Do credit checking here
		String creditStatus = new String(_creditSystem.checkCreditCard(orderid, creditCardNumber, creditCardType, totalCharge));
			
            TextMessage replyMessage = _Session.createTextMessage();
            replyMessage.setText(creditStatus);
            System.out.println("JMS Credit CheckAdapter : Sending reply message : " + orderid);                  			    
                
            if(_debug.equalsIgnoreCase("TRUE"))
                System.out.println("MESSAGE : " + replyMessage);

            // Step 10: Send reply message
            Destination tempDest = message.getJMSReplyTo();
			_replySender.send(tempDest,replyMessage); 
    			    
            if(_debug.equalsIgnoreCase("TRUE"))
			    System.out.println("JMS Credit CheckAdapter : Inside receiveAndReply : Finished");    			    
		}
    }
 
    public void closeConnection()
    {
        try
        {
            if (_Connection!=null)
            {
                // Step 11: Close connection
                _Connection.close();
            }
        }
        catch (JMSException e)
        {
            e.printStackTrace();
        }
    }
    
    public static void main(String[] args)
    {
        CreditCheckAdapter replier = null;
        try
        {
             replier = new CreditCheckAdapter(args[0]);
             replier.init();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            replier.closeConnection();
        }
    }
}