/* 
 * Lab G Exercise 1: Inventory Check adapter
 * Steps 1 to 11.
 */

/*
 * InventoryCheckAdapter.java
 * DESCRIPTION : Interfaces between JMS Business Process Manager and Inventory Database
 *               Check the inventory to fulfill user request using following steps:
 *               Receives message on Destination 
 *               Parses the input xml message
 *               Executes stored procedure to check inventory using JDBC to connect to database
 *               Creates XML JMS message from the output of stored procedure
 *               Sends reply back to replyDest 
 *
 */

import com.tibco.tibchoc.*;

// Step 1: Import the entire javax.jms package
import javax.jms.*;

public class InventoryCheckAdapter
{
    private String _propertiesFileName;
    private String _serverName;
    private String _debug;
    private String _receiveDestinationName;
    private String _user;
    private String _password;
    private String _factoryName;
  
    private Connection _Connection = null;
    private Session _Session;
    private MessageConsumer _Receiver;
    private MessageProducer _replySender;
    
    private DBConnection _dbConnection;

    public InventoryCheckAdapter (String fileName) throws Exception
    {
        _propertiesFileName = fileName;
    }

    public void init() throws Exception 
    {
        System.out.println("JMS Inventory Check adapter : Starting up....");
        AppProperties.loadProperties(_propertiesFileName);
        _serverName             = AppProperties.getProperty("TIBCHOC.serverName");
        _debug                  = AppProperties.getProperty("TIBCHOC.debug");
        _user                   = AppProperties.getProperty("TIBCHOC.user");
        _password               = AppProperties.getProperty("TIBCHOC.password");
        _factoryName            = AppProperties.getProperty("TIBCHOC.factory");
        _receiveDestinationName = AppProperties.getProperty("TIBCHOC.receiveDestination");
        _dbConnection           = new DBConnection(_propertiesFileName);
        _dbConnection.setup();
        openJMSConnection();
    }
    
    public void openJMSConnection() throws Exception
    {
        System.out.println("JMS Inventory CheckAdapter : OpenJMSConnection : [" + _serverName + "] Starting up....");
        System.out.println("JMS Inventory CheckAdapter : Receive destination name : " + _receiveDestinationName);
                
        TibjmsUtilities.initJNDI(_serverName);
        
        // Step 2: Use JNDI to obtain a connection factory
        ConnectionFactory factory = (ConnectionFactory)TibjmsUtilities.lookup(_factoryName);
        
        // Step 3: Create a connection using the connection factory
        _Connection = factory.createConnection(_user,_password);
        
        // Step 4: Create a session using the connection
        _Session = _Connection.createSession(false,Session.AUTO_ACKNOWLEDGE);
        
        // Step 5: Obtain a specific destination
        Destination dest = (Destination)TibjmsUtilities.lookup(_receiveDestinationName);
        
        // Step 6: Create a consumer
        _Receiver = _Session.createConsumer(dest);
        
        // Step 7: Create a producer
        _replySender = _Session.createProducer(null);
            
        // Step 8: Start a connection
        _Connection.start();
        for(;;) receiveAndReply();
    }
        
        
    public void receiveAndReply() throws Exception
    {
        if(_debug.equalsIgnoreCase("TRUE"))
            System.out.println("JMS Inventory CheckAdapter : Inside receiveAndReply : Waiting for messages");

        // Step 9: Receive message
        TextMessage message =  (TextMessage)_Receiver.receive();
        System.out.println("JMS Inventory CheckAdapter : Received message" );
            
        if(_debug.equalsIgnoreCase("TRUE"))
            System.out.println("MESSAGE : " + message);
            
            
        if (message.getJMSReplyTo() == null)
			System.out.println("JMS Inventory CheckAdapter : Error : ReplyTo is null");
		else
		{
			//Inventory Check
			//Parse message to get itemNames and itemQuantities
            DomDoc msgWeb = new DomDoc(((TextMessage)message).getText());
            msgWeb.parse();
            String orderid = msgWeb.get("OrderID");
            System.out.println("JMS Inventory CheckAdapter : Received message : " + orderid);
            int NumberOfItems = msgWeb.getLength("Items");
            String items = "";
            String quantity = "";
            for(int i=0; i< NumberOfItems; i++)
            {
                if (i>0)
                {
                    items += "|";
                    quantity += "|";
                }
                items += msgWeb.getNthElement("ItemName", i);
                quantity += msgWeb.getNthElement("Quantity", i);
            }
            if(_debug.equalsIgnoreCase("TRUE"))
            {
                System.out.println("JMS Inventory CheckAdapter : Items : " + items);
                System.out.println("JMS Inventory CheckAdapter : Quantity : " + quantity);
            }
            
			//Execute stored procedure
            java.sql.Connection conn = _dbConnection.getConnection();
            java.sql.CallableStatement cstmt = conn.prepareCall
                ("{call inventory_data_check (?,?,?,?)}");
            cstmt.setString(1,items);
            cstmt.setString(2,quantity);
            cstmt.registerOutParameter(3, java.sql.Types.LONGVARCHAR);
            cstmt.registerOutParameter(4, java.sql.Types.LONGVARCHAR);
            cstmt.execute();
            _dbConnection.freeConnection(conn);
            String Status = cstmt.getString(3);
            String Detail = cstmt.getString(4);
            
            //create reply meessage
            DomDocOut iDom = new DomDocOut();
            iDom.createRoot("Reply");
            iDom.add("OrderID", orderid);
            iDom.add("Status",Status);
            iDom.add("Detail",Detail);
            TextMessage replyMessage = _Session.createTextMessage();
            replyMessage.setText(iDom.toString());
            System.out.println("JMS Inventory CheckAdapter : Sending reply message : " + orderid);
                
            if(_debug.equalsIgnoreCase("TRUE"))
                System.out.println("MESSAGE : " + iDom);
               
            //send reply message
            
            // Step 10: Send reply message
            Destination tempDest = (Destination)message.getJMSReplyTo();
			_replySender.send(tempDest,replyMessage);  
                
            if(_debug.equalsIgnoreCase("TRUE"))
			    System.out.println("JMS Inventory CheckAdapter : Inside receiveAndReply : Finished");
		}
    }

    public void closeConnection()
    {
        try
        {
            if (_Connection!=null)
            {
                // Step 11: Close connection
                _Connection.close();
            }
        }
        catch(JMSException e)
        {
            e.printStackTrace();
        }
    }
    
    public static void main(String args[])
    {
        InventoryCheckAdapter replier = null;
        try 
        {
             replier = new InventoryCheckAdapter(args[0]);
             replier.init();
        }
        catch (Exception e) 
        {
            e.printStackTrace();
            replier.closeConnection();
        }
    }
}

