/* 
 * Lab F: Simplified version of BPC
 * Steps 1 to 11.
 */
 
/*
 * BusinessProcessCoordinator.java
 * DESCRIPTION : Acts as Business Process Manager by coordinating following activites
 *               1. Receives Message From Servlet
 *               2. Sends asynchronous request to Credit Check Service
 *               3. Sends asynchronous request to Inventory Check Service
 *               4. When both the replies are received, sends reply back to servlet
 *
 */

import com.tibco.tibchoc.AppProperties;
import com.tibco.tibchoc.*;

// Step 1: Import the entire javax.jms package
import javax.jms.*;


public class BusinessProcessCoordinator implements MessageListener
{
    private String _propertiesFileName;
    private String _serverName;
    private String _debug;
    private String _user;
    private String _password;
    private String _factoryName;
    
    private ConnectionFactory _Factory;
    private Connection _Connection;
    
    private Session    _Session;
    
    private Destination       _webReceiveDestination;
                    
    private MessageConsumer   _webReceiver;
    private MessageProducer   _webReplier;
    
    public BusinessProcessCoordinator (String fileName) throws Exception
    {
        _propertiesFileName = fileName;
    }

    public void init() throws Exception 
    {
        System.out.println("Business Process Manager : init ");
        AppProperties.loadProperties(_propertiesFileName);
        _serverName    = AppProperties.getProperty("TIBCHOC.serverName");
        _debug         = AppProperties.getProperty("TIBCHOC.debug");
        _user          = AppProperties.getProperty("TIBCHOC.user");
        _password      = AppProperties.getProperty("TIBCHOC.password");
        _factoryName   = AppProperties.getProperty("TIBCHOC.factory");

        openJMSConnection();
    }
    
    public void initialize()
    {
        //initialization of Connection Factory and creation of connection
        try
        {
            TibjmsUtilities.initJNDI(_serverName);
            
            // Step 2: Use JNDI to obtain a connection factory
            _Factory = (ConnectionFactory)TibjmsUtilities.lookup(_factoryName);
            
            // Step 3: Create a connection using the connection factory
            _Connection = _Factory.createConnection(_user, _password);
            
            // Step 4: Create a session using the connection
            _Session = _Connection.createSession(false,Session.AUTO_ACKNOWLEDGE);            
        }
        catch (Exception e) 
        {
            System.out.println(e);
            closeAll();
            System.exit(0);
        }
    }   
       
    public void openJMSConnection()
    {
        try 
        {
            System.out.println("JBPC : openJMSConnection ");
            initialize();
            
            //To receive messages from servlet and reply back

            // Step 5: Obtain a specific destination
            _webReceiveDestination = (Destination)TibjmsUtilities.lookup(AppProperties.getProperty("TIBCHOC.WEB2JBPC"));
            
            // Step 6: Create a message Consumer
            _webReceiver    = _Session.createConsumer(_webReceiveDestination);
            _webReceiver.setMessageListener(this);
	      System.out.println("JBPC : Receiver created : WEB2BPC : "+ AppProperties.getProperty("TIBCHOC.WEB2JBPC"));
            
            // Step 7: Create a message producer
            _webReplier     = _Session.createProducer(null);            
            
            
            // Step 8: Start a connection
            _Connection.start();
            
             synchronized (this) 
             {
                try 
                {
                    this.wait();
                } 
                catch (InterruptedException ie) {}
             }
            
        }
        catch(Exception e)
        { 
            System.out.println(e);
            closeAll();
            System.exit(0);
        }
    }
        
    // Step 9: Receive aysnchronously delivered messages
    public void onMessage(Message message) 
    {
        System.out.println("JBPC : Received Message From Servlet");
        if(_debug.equalsIgnoreCase("TRUE"))
            System.out.println("Message :  "+ message);

        try
        {
            //Parse message            
            DomDoc msgFromServlet = new DomDoc(((TextMessage)message).getText());
            msgFromServlet.parse();
            String orderid = msgFromServlet.get("OrderID");
            System.out.println("JBPC : Order Id :  "+ orderid);
            
            //create dummy reply
            DomDocOut webReplyDoc = new DomDocOut();
            webReplyDoc.createRoot("Order");
            webReplyDoc.add("OrderID",orderid);
            webReplyDoc.add("Status", "SUCCESS");
            webReplyDoc.add("Detail", " ");
            TextMessage wMessage = _Session.createTextMessage();
            wMessage.setText(webReplyDoc.toString());
            
            //send reply to servlet
            
            // Step 10: Send reply message
            Destination replyToDestination = message.getJMSReplyTo();
            _webReplier.send(replyToDestination, wMessage);
            
            
        } 
        catch(Exception e) 
        {
            System.out.println(e);
            closeAll();
            System.exit(0);
        }
    }
       
    private void closeAll()
    {
        try
        {
            if (_Connection!=null)
            {
                // Step 11: Close the session and connection
                _Connection.close();
                _Connection = null;
            }
        }
        catch(JMSException e)
        {
            System.out.println(e);
        }
    }

    public static void main (String[] args)
    {
        try{
            BusinessProcessCoordinator bpc = new BusinessProcessCoordinator(args[0]);
            bpc.init();
        } catch (Exception e){
            System.out.println(e);
            System.exit(0);
        }
    } 
}
 